/**
 * @file Experiment.java
 * 
 * @date Aug 4, 2004
 * @author Alexandre Bouchard
 */
package ca.mcgill.rl.apps;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import ca.mcgill.rl.ndp.Action;
import ca.mcgill.rl.ndp.Agent;
import ca.mcgill.rl.ndp.EnvironmentSimulator;
import ca.mcgill.rl.util.Configurable;
import ca.mcgill.rl.util.Gaugeable;
import ca.mcgill.rl.util.TypedProperties;

/**
 *
 */
public class Experiment implements Configurable
{
    /**
     * TODO:
     * Comment for <code>agent</code>
     */
    private Agent agent;
    
    /**
     * TODO:
     * Comment for <code>environment</code>
     */
    private EnvironmentSimulator environment;

    /**
     * TODO:
     * Comment for <code>numberOfIterations</code>
     */
    private int numberOfIterations;

    /**
     * TODO:
     * Comment for <code>greedyTrajectoryLength</code>
     */
    private int greedyTrajectoryLength;

    /**
     * TODO:
     * Comment for <code>learningTrajectoryLength</code>
     */
    private int learningTrajectoryLength;
    
    /**
     * TODO:
     * Comment for <code>policyTestingFrequency</code>
     */
    private int policyTestingFrequency;
    
    /**
     * TODO:
     * Comment for <code>discountFactor</code>
     */
    private int discountFactor;
    
    /**
     * TODO:
     * Comment for <code>propertyPrefix</code>
     */
    private static String propertyPrefix = "Experiment.";

    /**
     * TODO:
     * Comment for <code>trajectoriesPerPolicyTest</code>
     */
    private int trajectoriesPerPolicyTest;
    
    /**
     * TODO:
     * Comment for <code>experimentDirectory</code>
     */
    private File experimentDirectory;
    
    /*
     * @param agent
     * @param environment
     * @param numberOfIterations
     * @param greedyTrajectoryLength
     * @param learningTrajectoryLength
     * @param policyTestingFrequency
     * @param discountFactor
     *
    public Experiment(	Agent agent, 
            				EnvironmentSimulator environment, 
            				int numberOfIterations, 
            				int greedyTrajectoryLength,
            				int learningTrajectoryLength,
            				int policyTestingFrequency,
            				int trajectoriesPerPolicyTest,
            				int discountFactor)
    {
        this.agent = agent;
        this.environment = environment;
        this.numberOfIterations = numberOfIterations;
        this.greedyTrajectoryLength = greedyTrajectoryLength;
        this.learningTrajectoryLength = learningTrajectoryLength;
        this.policyTestingFrequency = policyTestingFrequency;
        this.trajectoriesPerPolicyTest = trajectoriesPerPolicyTest;
        this.discountFactor = discountFactor;
    }*/
    
    /**
     * @param experimentDirectory
     */
    public Experiment(File experimentDirectory)
    {
        this.experimentDirectory = experimentDirectory;
    }
    
    /**
     * 
     */
    public void startExperiment()
    {
        try
        {
            PrintWriter out = new PrintWriter(new FileOutputStream(new File(experimentDirectory, "totalReturn.hist")));
            System.out.println("Iteration \t Averaged return");
            for (int i = 0; i < numberOfIterations; i++)
            {
                // Test periodically
                if (i % policyTestingFrequency == 0)
                {
                    double totalReward = 0;
                    for (int k = 0; k < trajectoriesPerPolicyTest; k++)
                    {
            			environment.setState(environment.getStateSpace().getRandomState());
            			agent.initialize();
            			double currentDiscount = 1;
            			for (int j = 0; j < greedyTrajectoryLength && !environment.isAtTerminalState(); j++)
            			{
            				Action selectedAction = agent.act();
            				environment.generateTransition(selectedAction);
            				totalReward = currentDiscount * environment.getReward() + totalReward;
            				currentDiscount = currentDiscount * discountFactor;
            			}
                    }
                    System.out.println("" + i + "\t" + (totalReward/trajectoriesPerPolicyTest));
                    out.println("" + i + "\t" + (totalReward/trajectoriesPerPolicyTest));
                    out.flush();
                     if (agent instanceof Gaugeable)
                     {
                         ((Gaugeable) agent).printStatistics(experimentDirectory);
                     }            
                }
                 
                // Learn
                environment.setState(environment.getStateSpace().getRandomState());
                agent.initialize();
                for (int j = 0; j < learningTrajectoryLength && !environment.isAtTerminalState(); j++)
                {
                    Action selectedAction = agent.actAndLearn();
                    environment.generateTransition(selectedAction);
                }
            }            
            out.close();
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /**
     * @param properties
     */
    public void configure(TypedProperties properties)
    {
        discountFactor = properties.getPropertyAsInt(propertyPrefix + "discountFactor");
        learningTrajectoryLength = properties.getPropertyAsInt(propertyPrefix + "learningTrajectoryLength");
        greedyTrajectoryLength = properties.getPropertyAsInt(propertyPrefix + "greedyTrajectoryLength");
        numberOfIterations = properties.getPropertyAsInt(propertyPrefix + "numberOfIterations");
        policyTestingFrequency = properties.getPropertyAsInt(propertyPrefix + "policyTestingFrequency");
        trajectoriesPerPolicyTest = properties.getPropertyAsInt(propertyPrefix + "trajectoriesPerPolicyTest");
        
        environment = (EnvironmentSimulator) properties.getPropertyAsObject(propertyPrefix + "environment");
        if (environment instanceof Configurable)
        {
            ((Configurable) environment).configure(properties);
        }
        
        agent = (Agent) properties.getPropertyAsObject(propertyPrefix + "agent");
        agent.setEnvironment(environment);
        agent.setDiscountFactor(discountFactor);
        if (agent instanceof Configurable)
        {
            ((Configurable) agent).configure(properties);
        }
    }
    
    /**
     * @param args
     */
    public static void main(String [] args)
    {
        if (args.length != 1)
        {
            throw new RuntimeException("You must specify the directory where the experiment parameters will be read and the results written.");
        }
        try
        {
            File experimentDirectory = new File(args[0]);
            File timeStampedExperimentDirectory = new File(experimentDirectory, new Date().toString());
            if (!timeStampedExperimentDirectory.isDirectory())
            {
                timeStampedExperimentDirectory.mkdir();
            }            
            Experiment experiment = new Experiment(timeStampedExperimentDirectory);
            File propertyFile = new File(experimentDirectory, "experimentParameters.properties");
            TypedProperties properties = new TypedProperties();
            properties.load(new FileInputStream(propertyFile));
            experiment.configure(properties);
            experiment.startExperiment();
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
