/**
 * @file 	SupervisedTest.java
 * 
 * @date 	Aug 12, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.apps;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;

import ca.mcgill.rl.ndp.Approximator;
import ca.mcgill.rl.util.AffineVector;
import ca.mcgill.rl.util.Configurable;
import ca.mcgill.rl.util.Gaugeable;
import ca.mcgill.rl.util.RealValuedFunction;
import ca.mcgill.rl.util.State;
import ca.mcgill.rl.util.TypedProperties;

/**
 *
 */
public class SupervisedTest implements Configurable
{
    /**
     * TODO:
     * Comment for <code>propertyPrefix</code>
     */
    private static String propertyPrefix = "SupervisedTest.";
    
    /**
     * TODO:
     * Comment for <code>targetFunction</code>
     */
    private RealValuedFunction targetFunction;
    
    /**
     * TODO:
     * Comment for <code>experimentDirectory</code>
     */
    private File experimentDirectory;

    /**
     * TODO:
     * Comment for <code>numberOfIterations</code>
     */
    private int numberOfIterations;

    /**
     * TODO:
     * Comment for <code>approximatorTestingFrequency</code>
     */
    private int approximatorTestingFrequency;

    /**
     * TODO:
     * Comment for <code>evaluationsPerApproximatorTesting</code>
     */
    private int evaluationsPerApproximatorTesting;

    /**
     * TODO:
     * Comment for <code>approximator</code>
     */
    private Approximator approximator;
    
    /**
     * @param experimentDirectory
     */
    public SupervisedTest(File experimentDirectory)
    {
        this.experimentDirectory = experimentDirectory;
    }  
    
    /**
     * 
     */
    public void startSupervisedTest()
    {   
        System.out.println("Iteration \t Averaged error");
        for (int j = 0; j < numberOfIterations; j++)
        {
            if (j % approximatorTestingFrequency == 0)
            {
	            double partialSumError = 0;
	            for (int i = 0; i < evaluationsPerApproximatorTesting; i++)
	            {
	                State randomState = targetFunction.getStateSpace().getRandomState();
	                double functionValue = targetFunction.evaluateAt(randomState);
	                double approximation = approximator.evaluateAt(randomState);
	                partialSumError = partialSumError + Math.abs((functionValue - approximation) 
	                        										/ functionValue); 
	            }
	            System.out.println("" + j + "\t" + (partialSumError/(double) evaluationsPerApproximatorTesting));
	            if (approximator instanceof Gaugeable)
	            {
	                ((Gaugeable) approximator).printStatistics(experimentDirectory);
	            }
            }
            
            State randomState = targetFunction.getStateSpace().getRandomState();
            double approximation = approximator.evaluateAt(randomState);
            AffineVector gradient = approximator.computeGradient(randomState);
            gradient.times(targetFunction.evaluateAt(randomState) - approximation);
            approximator.updateParameters(gradient);
        }        
    }
    
    /**
     * @param properties
     */
    public void configure(TypedProperties properties)
    {
        approximatorTestingFrequency = properties.getPropertyAsInt(propertyPrefix + "approximatorTestingFrequency");
        evaluationsPerApproximatorTesting = properties.getPropertyAsInt(propertyPrefix + "evaluationsPerApproximatorTesting");
        numberOfIterations = properties.getPropertyAsInt(propertyPrefix + "numberOfIterations");
        
        targetFunction = (RealValuedFunction) properties.getPropertyAsObject(propertyPrefix + "targetFunction");
        if (targetFunction instanceof Configurable)
        {
            ((Configurable) targetFunction).configure(properties);
        }
        
        
        approximator = (Approximator) properties.getPropertyAsObject(propertyPrefix + "approximator");
        approximator.setStateSpace(targetFunction.getStateSpace());
        if (approximator instanceof Configurable)
        {
            ((Configurable) approximator).configure(properties);
        }
    }
    
    /**
     * @param args
     */
    public static void main(String[] args)
    {
        if (args.length != 1)
        {
            throw new RuntimeException("You must specify the directory where the experiment parameters will be read and the results written.");
        }
        try
        {
            File supervisedTestDirectory = new File(args[0]);
            SupervisedTest supervisedTest = new SupervisedTest(supervisedTestDirectory);
            File propertyFile = new File(supervisedTestDirectory, "supervisedTestParameters.properties");
            TypedProperties properties = new TypedProperties();
            properties.load(new FileInputStream(propertyFile));
            supervisedTest.configure(properties);
            supervisedTest.startSupervisedTest();
        }
        catch (FileNotFoundException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (IOException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }
}
