/**
 * @file EnvironmentSimulator.java
 * 
 * @date Aug 4, 2004
 * @author Alexandre Bouchard
 */

package ca.mcgill.rl.ndp;

import java.util.List;

import ca.mcgill.rl.util.State;
import ca.mcgill.rl.util.StateSpace;

/**
 * An environment simulator in NDP (i.e. a NDP problem).
 * 
 * See http://www-anw.cs.umass.edu/~rich/book/3/node2.html#SECTION00110000000000000000
 */
public interface EnvironmentSimulator
{
    /**
     * Returns the current state of the simulator.
     * 
     * @return The current state of the simulator.
     */
    public State getState();
    
    /**
     * Sets the current state to the given state.
     * 
     * @param state
     */
    public void setState(State state);
    
    /**
     * Returns the reward corresponding to the last call of generateTransition().
     * 
     * @return The reward corresponding to the last call of generateTransition().
     */
    public double getReward();
    
    /**
     * Generates a transition given that selectedAction is chosen.
     * 
     * @param selectedAction
     */
    public void generateTransition(Action selectedAction);
    
    /**
     * Returns the actions that are possible from the current state.
     * 
     * @return The actions that are possible from the current state.
     */
    public List getPossibleActions();

    /**
     * Is the current state terminal?
     * 
     * @return Is the current state terminal?
     */
    public boolean isAtTerminalState();

    /**
     * @return
     */
    public StateSpace getStateSpace();
}
