/**
 * @file 	MountainCarHeuristicPolicy.java
 * 
 * @date 	Aug 17, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.ndp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import ca.mcgill.rl.util.DiscreteMeasure;
import ca.mcgill.rl.util.EuclideanLocation;
import ca.mcgill.rl.util.State;

/**
 *
 */
public class MountainCarHeuristicPolicy implements HeuristicPolicy
{
    /**
     * TODO:
     * Comment for <code>environment</code>
     */
    private TransitionProbabilities transitionProbabilities;
    
    /**
     * TODO:
     * Comment for <code>randomNumberGenerator</code>
     */
    private Random randomNumberGenerator = new Random(System.currentTimeMillis());    

    /**
     * @param environment
     */
    public void setEnvironment(EnvironmentSimulator environment)
    {
        this.transitionProbabilities = ((MountainCarEnvironment) environment).getTransitionProbabilities();
    }

    /**
     * @param currentState
     * @param possibleActions
     * @return
     */
    public Action selectAction(State currentState, List possibleActions)
    {
        if (possibleActions.isEmpty())
        {
            throw new RuntimeException("No action available at current state " + currentState.toString());
        }
        // 1- Find the subset of possibleActions with the highest value according to the approximator
        List optimalActions = new ArrayList();
        Iterator possibleActionsIterator = possibleActions.iterator();
        double maxValue = -Double.MAX_VALUE;
        while (possibleActionsIterator.hasNext())
        {
            Action currentAction = (Action) possibleActionsIterator.next();
            DiscreteMeasure measure = (DiscreteMeasure) transitionProbabilities.getTransitionProbabilityMeasure(currentState, currentAction);
            EuclideanLocation nextLocation = (EuclideanLocation) measure.getNonZeroProbabilityStates().get(0);
            double nextPosition = nextLocation.getCoordinate(0);
            double nextVelocity = nextLocation.getCoordinate(1);
            double currentValue = 0.5 * nextVelocity * nextVelocity + 0.0025 * Math.sin(3*nextPosition) / 3;
            //double currentValue = 
            if (currentValue > maxValue)
            {
                optimalActions.clear();
                optimalActions.add(currentAction);
                maxValue = currentValue;
            }
            if (currentValue == maxValue)
            {
                optimalActions.add(currentAction);
            }
        }
        // 2- Pick an action from this subset randomly (uniform distribution)
        return (Action) optimalActions.get(randomNumberGenerator.nextInt(optimalActions.size()));
    }
}
