/**
 * @file 	SimpleImplicitCostToGoPolicy.java
 * 
 * @date 	Aug 17, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.ndp;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Random;

import ca.mcgill.rl.util.State;

/**
 *
 */
public class SimpleImplicitCostToGoPolicy implements ImplicitCostToGoPolicy
{
    /**
     * TODO:
     * Comment for <code>approximator</code>
     */
    private Approximator approximator;
    
    /**
     * TODO:
     * Comment for <code>transitionProbabilities</code>
     */
    private TransitionProbabilities transitionProbabilities;

    /**
     * @param approximator
     */
    public void setApproximator(Approximator approximator)
    {
        this.approximator = approximator;
    }
    
    /**
     * TODO:
     * Comment for <code>randomNumberGenerator</code>
     */
    private Random randomNumberGenerator = new Random(System.currentTimeMillis());

    /**
     * @param transitionProbabilities
     */
    public void setTransitionProbabilities(
            TransitionProbabilities transitionProbabilities)
    {
        this.transitionProbabilities = transitionProbabilities;
    }

    /**
     * @param currentState
     * @param possibleActions
     * @return
     */
    public Action selectAction(State currentState, List possibleActions)
    {
        if (possibleActions.isEmpty())
        {
            throw new RuntimeException("No action available at current state " + currentState.toString());
        }
        // 1- Find the subset of possibleActions with the highest value according to the approximator
        List optimalActions = new ArrayList();
        Iterator possibleActionsIterator = possibleActions.iterator();
        double maxValue = -Double.MAX_VALUE;
        while (possibleActionsIterator.hasNext())
        {
            Action currentAction = (Action) possibleActionsIterator.next();
            double currentValue = transitionProbabilities.getTransitionProbabilityMeasure(currentState, currentAction).integrate(approximator);
            if (currentValue > maxValue)
            {
                optimalActions.clear();
                optimalActions.add(currentAction);
                maxValue = currentValue;
            }
            if (currentValue == maxValue)
            {
                optimalActions.add(currentAction);
            }
        }
        // 2- Pick an action from this subset randomly (uniform distribution)
        return (Action) optimalActions.get(randomNumberGenerator.nextInt(optimalActions.size()));
    }

}
