/**
 * @file 	ActiveLocations.java
 * 
 * @date 	Aug 8, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.sdm;

import java.util.Collection;

import ca.mcgill.rl.util.EuclideanLocation;

/**
 * The class ActiveLocations represents the collection of hard location that are
 * activated by a given location. 
 * 
 * That is, the subset of the hard locations that have a positive similarity measure 
 * with the given location. Since SDM is a local architecture, this correspond to a
 * small neighborhood (relative to the space) around the center.
 */
public class ActiveLocations
{
    /**
     * The underlying collection of hard locations.
     */
    private Collection activeLocations;
    
    /**
     * The center of this neighborhood.
     */
    private EuclideanLocation center;
    
    /**
     * Creates an instance of ActiveLocations.
     * 
     * Assumes that the given collection contains hard locations and that these hard
     * locations are precisely those with a positive similarity measure with the
     * center.
     * 
     * @param center
     * @param activeLocations
     */
    public ActiveLocations(EuclideanLocation center, Collection activeLocations)
    {
        this.center = center;
        this.activeLocations = activeLocations;
    }
    
    /**
     * Returns the center.
     * 
     * @return The center.
     */
    public EuclideanLocation getCenter()
    {
        return center;
    }
    
    /**
     * Returns the collection of hard locations contained in this neighborhood.
     * 
     * @return The collection of hard locations contained in this neighborhood.
     */
    public Collection getHardLocations()
    {
        return activeLocations;
    }
}
