/**
 * @file 	HardLocation.java
 * 
 * @date 	Aug 9, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.sdm;

import ca.mcgill.rl.util.EuclideanLocation;

/**
 * An interface for hard locations in a SDM architecture.
 */
public interface HardLocation
{
    /**
     * Returns the position in the space of this hard location.
     * 
     * @return
     */
    EuclideanLocation getPosition();
    
    /**
     * Compute the similarity with a given location according to the implementation
     * similarity metric.
     * 
     * A similarity metric mu(.,.) is a map from the state set to [0,1]. We assume
     * that (1 - mu) is a metric on the space.
     * 
     * @param location
     * @return
     */
    double similarityWith(EuclideanLocation location);

    /**
     * Returns the identifier of this hard location.
     * 
     * @return
     */
    int getId();
}
