/**
 * @file 	SparseVector.java
 * 
 * @date 	Aug 21, 2004
 * @author 	Alexandre Bouchard
 */

package ca.mcgill.rl.util;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *
 */
public class SparseVector implements AffineVector
{
    /**
     * TODO:
     * Comment for <code>value</code>
     */
    private Map values = new HashMap();
    
    /**
     * TODO:
     * Comment for <code>dimensionality</code>
     */
    private int dimensionality;
    
    /**
     * Initializes a StandardVector with the given dimensionality for the enclosing
     * vector space.
     */
    public SparseVector(int dimensionality)
    {
        this.dimensionality = dimensionality;
    } 
    
    public Iterator getNonZeroComponents()
    {
        return values.entrySet().iterator();
    }

    /**
     * @param v2
     */
    public void add(AffineVector v2)
    {
        if (v2 instanceof SparseVector)
        {
            SparseVector sv2 = (SparseVector) v2;
            Iterator nonZeroElements = sv2.getNonZeroComponents();
            while (nonZeroElements.hasNext())
            {
                Map.Entry entry = (Map.Entry) nonZeroElements.next();
                int key = ((Integer) entry.getKey()).intValue();
                double value = ((Double) entry.getValue()).doubleValue();
                setComponent(key, getComponent(key) + value);
            }
        }
        else
        {
            for (int i = 0; i < dimensionality; i++)
            { 
                setComponent(i, getComponent(i) + v2.getComponent(i));
            }            
        }
    }

    /**
     * 
     */
    public void opposite()
    {
        Iterator iterator = getNonZeroComponents();
        while (iterator.hasNext())
        {
            Map.Entry entry = (Map.Entry) iterator.next();
            double currentValue = ((Double) entry.getValue()).doubleValue();
            entry.setValue(new Double(-currentValue));
        }
    }

    /**
     * @param scalar
     */
    public void times(double scalar)
    {
        Iterator iterator = getNonZeroComponents();
        while (iterator.hasNext())
        {
            Map.Entry entry = (Map.Entry) iterator.next();
            double currentValue = ((Double) entry.getValue()).doubleValue();
            entry.setValue(new Double(scalar * currentValue));
        }
    }

    /**
     * @param index
     * @return
     */
    public double getComponent(int index)
    {
        Double value = (Double) values.get(new Integer(index));
        if (value == null)
        {
            return 0;
        }
        else
        {
            return value.doubleValue();
        }
    }

    /**
     * @param index
     * @param value
     */
    public void setComponent(int index, double value)
    {
        if (value == 0)
        {
            values.remove(new Integer(index));
        }
        else
        {
            values.put(new Integer(index), new Double(value));
        }
    }

    /**
     * @return
     */
    public int getDimensionality()
    {
        return dimensionality;
    }

    /**
     * @param newDimensionality
     */
    public void setDimensionality(int newDimensionality)
    {
        this.dimensionality = newDimensionality;
    }

}
