/**
 * @file Agent.java
 * 
 * @date Aug 4, 2004
 * @author Alexandre Bouchard
 */

package ca.mcgill.rl.ndp;

/**
 * An interface for agents in NDP problems (i.e. NDP problem solvers).
 * 
 * See http://www-anw.cs.umass.edu/~rich/book/3/node2.html#SECTION00110000000000000000
 */

public interface Agent 
{
    /**
     * Prepare the agent for a new trajectory of learning,
     */
    public void initialize();
    
    /**
     * Given a state and set of actions that are possible from that state, returns
     * the actions that seems optimal to the agent.
     * 
     * @return The recommended action
     */
    public Action act();
    
    /**
     * Make the agent learn from a transition.
     * 
     * Again, the state from which the decision is given, together with a set of
     * actions that are possible from that state. Moreover, the reward collected for
     * the last transition is also given.
     * 
     * This method should be called many times sequentially, one call for each 
     * transition of a trajectory (by the class Experiment for instance). It is
     * assumed that initialize() is called before switching to another 
     * exploratory trajectory or before switching from a trajectory of exploitation
     * of the policy (act()) to a trajectory of learning (actAndLearn()).
     * 
     * @return The recommended action
     */
    public Action actAndLearn();
    
    /**
     * @param discountFactor
     */
    public void setDiscountFactor(double discountFactor);

    /**
     * @param environment
     */
    public void setEnvironment(EnvironmentSimulator environment);
}
